# -*- coding: utf-8 -*-
import sqlite3
import time, os, json

# User-defined parameters
db_file_path = "/linkedge/run/fc-runtime-data/"
db_file_name = "device-data-sqlite3.db"
table_max_entries = 10000  # Maximum entries per table
table_retain_count = 6000  # Entries retained in cleanup when data reaches the table_max_entries

if os.path.exists(db_file_path) == False:
    os.makedirs(db_file_path)

connect = sqlite3.connect(db_file_path + db_file_name)
cursor = connect.cursor()


def db_init(table_name):
    # Delete table when you want to recreate it
    # connect.execute('drop table if exists '+ table_name)
    sql = "CREATE TABLE IF NOT EXISTS " + table_name + "(KEY TEXT not null, VALUE REAL not null, TIMESTAMP NOT NULL DEFAULT (datetime('now','localtime')))"
    cursor.execute(sql)

    # Delete trigger when you want to update it
    # cursor.execute('DROP TRIGGER if exists db_clean_'+table_name)
    sql = '''CREATE TRIGGER if not exists db_clean_TableName2Replace AFTER INSERT ON TableName2Replace WHEN (select count(*) from TableName2Replace)>TableMaxEntry2Replace
  BEGIN
    DELETE FROM TableName2Replace WHERE rowid IN  (SELECT rowid FROM TableName2Replace ORDER BY TIMESTAMP ASC limit (select count(*) -TableRetainNum2Replace from TableName2Replace ));
  END;'''
    sql1 = sql.replace('TableName2Replace', table_name)
    sql = sql1.replace('TableMaxEntry2Replace', str(table_max_entries))
    sql1 = sql.replace('TableRetainNum2Replace', str(table_retain_count))
    cursor.execute(sql1)


def handler(event, context):
    print("-- Receive new event: ", event)
    try:
        edict = json.loads(event)
        topic = edict['topic']
        tlist = topic.split('/')
        if tlist[1] == 'sys':
            productKey = tlist[2]
            deviceName = tlist[3]
            tableName = productKey + '_' + deviceName
            print("-- Table Name : " + tableName)
            db_init(tableName)
        else:
            print("-- [Warn] Event topic should start with /sys/. Ignore!")
            return 'OK'

        payload = json.loads(edict['payload'])
        for key, value in payload.items():
            # You could add the data filetering logic here, before saving to DB.
            print("-- Save " + str(key) + ':' + str(value['value']) + " to table " + tableName)
            cursor.execute("INSERT OR REPLACE INTO " + tableName + " (KEY,VALUE) VALUES (?,?)",
                           (str(key), str(value['value'])))

        # [Optional] Show the total number of entries in the table
        cursor.execute("SELECT count(*) FROM " + tableName)
        print("-- Table {} total items = {}".format(tableName, cursor.fetchone()[0]))

        connect.commit()
    except Exception as e:
        print("-- [Warn] Event parse failed. Ignore! ", e)

    return 'OK'
