#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# ====#====#====#====
# author = jys
# file: ut_configErrCode
# time: 6/8/18
# ====#====#====#====

from leda_python.mbus import MbusBase
from gi.repository import GLib
import dbus.mainloop.glib
import logging
import threading
import signal
import time
import os

DMP_CM_WKN = 'iot.dmp.configmanager'
DMP_CM_OBJ_PATH = '/iot/dmp/configmanager'
DMP_CM_INTERFACE = DMP_CM_WKN

_logger = logging.getLogger(__name__)


def createThread(task, args=()):
    t = threading.Thread(target=task, args=args)
    t.setDaemon(True)
    t.start()


class MbusObject(dbus.service.Object):
    def __init__(self, conn, objPath):
        super(MbusObject, self).__init__(conn, objPath)

    @dbus.service.method(DMP_CM_INTERFACE, in_signature='s', out_signature='is')
    def get_config(self, key):
        _logger.debug("get_config: key: %s", key)

        return -10002001, ''

    @dbus.service.method(DMP_CM_INTERFACE, in_signature='ss', out_signature='i')
    def set_config(self, key, value):
        _logger.debug("set_config: key: %s, value: %s", key, value)

        return -10002004

    @dbus.service.method(DMP_CM_INTERFACE, in_signature='ssi', out_signature='i')
    def subscribe_config(self, wkn, key, value):
        _logger.debug("subscribe_config: wkn: %s, key: %s, value: %s", wkn, key, value)

        return -10002101


g_Flag = False


def Task_mainLoop():
    mainloop = GLib.MainLoop()
    mainloop.run()


def Signal_handle(signum, frame):
    global g_Flag
    print('recv a signal: %s' % (frame))
    g_Flag = True


if (__name__ == "__main__"):
    os.system(
        "ps -ef | grep \"config-manager\" | grep -v \"grep\" | awk \'{print $2}\' | xargs kill -9 1>> /dev/null 2>>/dev/null")
    logging.basicConfig(level=logging.DEBUG)
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    mbusConnectObject = MbusBase(DMP_CM_WKN)
    bus = mbusConnectObject.getBus()

    instance = MbusObject(bus, DMP_CM_OBJ_PATH)
    createThread(Task_mainLoop)
    signal.signal(signal.SIGINT, Signal_handle)
    while (1):
        time.sleep(0.5)
        if (g_Flag): break
